from holour.msg import Vector3


class ForceTorque:

    def __init__(self, force: Vector3, torque: Vector3, _type: str = ''):
        self._type = 'force_torque'
        self.force = force
        self.torque = torque

    def __eq__(self, other: object) -> bool:
        if isinstance(other, ForceTorque):
            return other.force == self.force and other.torque == self.torque
        return False

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self):
        return f"ForceTorque<force={self.force}, torque={self.torque}>"




class MQTTConfig:

    def __init__(self,
                 client_id: str,
                 address: str = "localhost",
                 port: int = 1883,
                 keep_alive: int = 60,
                 prefix: str = '/'):
        self.address = address
        self.port = port
        self.keep_alive = keep_alive
        self.prefix = prefix.rstrip('/')
        self.client_id = client_id



import json
from typing import Any

from holour.msg import *


class JsonEncoder(json.JSONEncoder):
    def default(self, obj: Any) -> Any:
        if isinstance(obj, Status):
            return obj.__dict__
        elif isinstance(obj, Vector3):
            return obj.__dict__
        elif isinstance(obj, Pose):
            return obj.__dict__
        elif isinstance(obj, Poses):
            return obj.__dict__
        elif isinstance(obj, ForceTorque):
            return obj.__dict__
        elif isinstance(obj, Script):
            return obj.__dict__
        elif isinstance(obj, WaypointConfig):
            return obj.__dict__
        elif isinstance(obj, Operation):
            return obj.__dict__
        elif isinstance(obj, Product):
            return obj.__dict__
        elif isinstance(obj, Task):
            return obj.__dict__
        else:
            return json.JSONEncoder.default(self, obj)


class JsonDecoder(json.JSONDecoder):

    def __init__(self, *args, **kwargs):
        json.JSONDecoder.__init__(self, object_hook=self.dict_to_object, *args, **kwargs)

    @staticmethod
    def dict_to_object(obj):
        if isinstance(obj, dict):
            if '_type' in obj and obj['_type'] == 'status':
                return Status(**obj)
            if '_type' in obj and obj['_type'] == 'vector3':
                return Vector3(**obj)
            if '_type' in obj and obj['_type'] == 'pose':
                return Pose(**obj)
            if '_type' in obj and obj['_type'] == 'poses':
                return Poses(**obj)
            if '_type' in obj and obj['_type'] == 'force_torque':
                return ForceTorque(**obj)
            if '_type' in obj and obj['_type'] == 'script':
                return Script(**obj)
            if '_type' in obj and obj['_type'] == 'waypoint_config':
                return WaypointConfig(**obj)
            if '_type' in obj and obj['_type'] == 'operation':
                return Operation(**obj)
            if '_type' in obj and obj['_type'] == 'product':
                return Product(**obj)
            if '_type' in obj and obj['_type'] == 'task':
                return Task(**obj)
        return obj


def json_encode(data: Any) -> str:
    return JsonEncoder().encode(data)


def json_decode(data: str) -> Any:
    return JsonDecoder().decode(data)





class Status:

    ONLINE = 'online'
    OFFLINE = 'offline'

    def __init__(self, id: str, ip: str, status: str, _type: str = ''):
        self._type = 'status'
        self.id = id
        self.ip = ip
        self.status = status

        assert self.status in [self.ONLINE, self.OFFLINE]

    def __eq__(self, other: object) -> bool:
        if isinstance(other, Status):
            return other.id == self.id and other.ip == self.ip and other.status == self.status
        return False

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self):
        return f"Status<id={self.id}, ip={self.ip}, status={self.status}>"



from unittest import TestCase

from holour import json_encode, json_decode
from holour.msg import Operation


class Test(TestCase):

    def test_operation(self):
        operation = Operation('uuid_1', 'product_1', {'x': 0.0, 'y': 0.0, 'z': 0.0, 'rx': 0.0, 'ry': 0.0, 'rz': 0.0})
        operation_string = json_encode(operation)
        expected_string = '{"_type": "operation", "uuid": "uuid_1", "product_uuid": "product_1", ' \
                          '"pose": {"x": 0.0, "y": 0.0, "z": 0.0, "rx": 0.0, "ry": 0.0, "rz": 0.0}, "description": ""}'

        assert type(operation_string) == str
        assert operation_string == expected_string, f"Expected {expected_string}, got: {operation_string}"

        operation_decoded = json_decode(operation_string)
        assert type(operation_decoded) == Operation, f"Got: {type(operation_decoded)}. Expected {Operation}"
        assert operation_decoded == operation, "The decoded object must be equal to the encoded"

    def test_operation_equals(self):
        op1 = Operation('uuid_1', 'product_1', {'x': 0.0, 'y': 0.0, 'z': 0.0, 'rx': 0.0, 'ry': 0.0, 'rz': 0.0})
        op2 = Operation('uuid_1', 'product_1', {'x': 0.0, 'y': 0.0, 'z': 0.0, 'rx': 0.0, 'ry': 0.0, 'rz': 0.0})
        op3 = Operation('uuid_2', 'product_1', {'x': 0.0, 'y': 0.0, 'z': 0.0, 'rx': 0.0, 'ry': 0.0, 'rz': 0.0})

        assert op1 == op2
        assert op1 != op3
        assert op1 != "not status"

    def test_operation_repr(self):
        operation = Operation('uuid_1', 'product_1', {'x': 0.0, 'y': 0.0, 'z': 0.0, 'rx': 0.0, 'ry': 0.0, 'rz': 0.0},
                              "Hard operation")
        expected, got = 'Hard operation', f'{operation}'

        assert expected in got, f"Expected {expected} in got: {got}"




from unittest import TestCase

from holour import json_encode, json_decode
from holour.msg import Task


class Test(TestCase):

    def test_task(self):
        task = Task('uuid_1', 'task_1', ['task_2', 'task_3'], [])
        task_string = json_encode(task)
        expected_string = '{"_type": "task", "uuid": "uuid_1", "name": "task_1", "sub_tasks": ["task_2", "task_3"], ' \
                          '"operations": [], "execute_parallel": false, "description": ""}'

        assert type(task_string) == str
        assert task_string == expected_string, f"Expected {expected_string}, got: {task_string}"

        task_decoded = json_decode(task_string)
        assert type(task_decoded) == Task, f"Got: {type(task_decoded)}. Expected {Task}"
        assert task_decoded == task, "The decoded object must be equal to the encoded"

    def test_task_equals(self):
        t1 = Task('uuid_1', 'task_1', ['task_2', 'task_3'], [])
        t2 = Task('uuid_1', 'task_1', ['task_2', 'task_3'], [])
        t3 = Task('uuid_2', 'task_1', ['task_2', 'task_3'], [])

        assert t1 == t2
        assert t1 != t3
        assert t1 != "not status"

    def test_task_repr(self):
        task = Task('uuid_1', 'task_1', ['task_2', 'task_3'], [])
        expected, got = 'task_1', f'{task}'

        assert expected in got, f"Expected {expected} in got: {got}"

    def test_sub_tasks_or_operations(self):
        with self.assertRaises(ValueError):
            task = Task('uuid_1', 'task_1', [], [])




class Product:

    def __init__(self, uuid: str, name: str, image: str, category: str, tags: [str] = None, part: bool = True,
                 description: str = "", _type: str = ''):
        assert type(uuid) == str
        assert type(name) == str
        assert type(image) == str
        assert type(category) == str
        assert type(tags) == list or tags is None
        assert type(part) == bool
        assert type(description) == str

        self._type = 'product'
        self.uuid = uuid
        self.name = name
        self.image = image
        self.category = category
        self.tags = tags if tags else []
        self.part = part
        self.description = description

    def __eq__(self, other: object) -> bool:
        if isinstance(other, Product):
            return other.uuid == self.uuid \
                   and other.name == self.name \
                   and other.image == self.image \
                   and other.category == self.category \
                   and other.tags == self.tags \
                   and other.part == self.part \
                   and other.description == self.description
        return False

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self):
        return f"<id={self.uuid},name={self.name},image={self.image},category={self.category},tags={self.tags}," \
               f"part={self.part},description={self.description}>"



import uuid as generate_uuid


class Task:

    def __init__(self, uuid: str, name: str, sub_tasks: [str], operations: [str], execute_parallel: bool = False,
                 description: str = "", _type: str = ''):
        """
        :param name: name of the task
        :param sub_tasks: list of sub-task uuid's
        :param operations: list of operation uuid's
        :param execute_parallel: whether the task can be executed in parallel (true/false)
        :param description: a description of the task
        :param uuid: UUID of the task
        """
        self._type = 'task'
        self.uuid = uuid if uuid != "" else str(generate_uuid.uuid4())
        self.name = name
        self.sub_tasks: [str] = sub_tasks
        self.operations: [str] = operations
        self.execute_parallel = execute_parallel
        self.description = description

        if len(sub_tasks) <= 0 and len(operations) <= 0:
            raise ValueError("A task must have a least 1 subtask or component!")

    def __eq__(self, other: object) -> bool:
        if isinstance(other, Task):
            return other.uuid == self.uuid and other.name == self.name
        return False

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self):
        return f"<id={self.uuid},name={self.name},sub_tasks={self.sub_tasks},operations={self.operations}," \
               f"execute_parallel={self.execute_parallel},description={self.description}>"



from unittest import TestCase

from holour import json_encode, json_decode
from holour.msg import Script, WaypointConfig


class Test(TestCase):

    def setUp(self) -> None:
        self.script = Script('test.script', "/")
        self.wp = WaypointConfig(show_waypoints=True, show_current_waypoint=True)

    def test_script_encode(self):
        script_string = json_encode(self.script)
        expected_string = '{"_type": "script", "filename": "test.script", "path": "/"}'

        assert type(script_string) == str, f"Got: {type(script_string)}. Expected {str}"
        assert script_string == expected_string, f"Expected {expected_string}, got: {script_string}"

    def test_script_decode(self):
        script_string = json_encode(self.script)
        script_decoded = json_decode(script_string)

        assert type(script_decoded) == Script, f"Got: {type(script_decoded)}. Expected {Script}"
        assert script_decoded == self.script, "Should be equal"

    def test_script_equals(self):
        same_script = Script('test.script', "/")
        diff_script = Script('test.script', "/new path")

        assert self.script == same_script
        assert self.script != diff_script
        assert self.script != "just a string"

    def test_script_repr(self):
        expected, got = 'test.script', f'{self.script}'
        assert expected in got, f"Expected {expected} in got: {got}"

    def test_waypoint_config_encode(self):
        wp_string = json_encode(self.wp)
        expected_string = '{"_type": "waypoint_config", "show_waypoints": true, "show_current_waypoint": true}'

        assert type(wp_string) == str, f"Got: {type(wp_string)}. Expected {str}"
        assert wp_string == expected_string, f"Expected {expected_string}, got: {wp_string}"

    def test_waypoint_config_decode(self):
        wp_string = json_encode(self.wp)
        wp_decoded = json_decode(wp_string)

        assert type(wp_decoded) == WaypointConfig, f"Got: {type(wp_decoded)}. Expected {WaypointConfig}"
        assert wp_decoded == self.wp, "Should be equal"

    def test_waypoint_config_equals(self):
        same_wp = WaypointConfig(True, True)
        diff_wp = WaypointConfig(False, True)

        assert self.wp == same_wp
        assert self.wp != diff_wp
        assert self.wp != "just a string"

    def test_waypoint_config_repr(self):
        expected, got = 'show_waypoints', f'{self.wp}'
        assert expected in got, f"Expected {expected} in got: {got}"



from unittest import TestCase

from holour import json_encode, json_decode
from holour.msg import Product


class Test(TestCase):

    def test_product(self):
        product = Product('uuid_1', 'product_1', "No image", "test")
        product_string = json_encode(product)
        expected_string = '{"_type": "product", "uuid": "uuid_1", "name": "product_1", ' \
                          '"image": "No image", "category": "test", "tags": [], "part": true, "description": ""}'

        assert type(product_string) == str
        assert product_string == expected_string, f"Expected {expected_string}, got: {product_string}"

        product_decoded = json_decode(product_string)
        assert type(product_decoded) == Product, f"Got: {type(product_decoded)}. Expected {Product}"
        assert product_decoded == product, "The decoded object must be equal to the encoded"

    def test_product_equals(self):
        p1 = Product('uuid_1', 'product_1', "No image", "test")
        p2 = Product('uuid_1', 'product_1', "No image", "test")
        p3 = Product('uuid_2', 'product_1', "No image", "test")

        assert p1 == p2
        assert p1 != p3
        assert p1 != "not status"

    def test_product_repr(self):
        product = Product('uuid_1', 'product_1', "No image", "test")
        expected, got = 'No image', f'{product}'

        assert expected in got, f"Expected {expected} in got: {got}"




class Operation:

    def __init__(self, uuid: str, product_uuid: str, pose: dict, description: str = "", _type: str = ''):
        assert type(uuid) == str
        assert type(product_uuid) == str
        assert type(pose) == dict
        assert type(description) == str
        for key in ['x', 'y', 'z', 'rx', 'ry', 'rz']:
            assert key in pose, f"The following must be in pose-dict: {key}. Pose: {pose}"
            assert type(pose[key]) == float, f"{key} must be of type float instead of: {type(pose[key])}."

        self._type = 'operation'
        self.uuid = uuid
        self.product_uuid = product_uuid
        self.pose = pose
        self.description = description

    def __eq__(self, other: object) -> bool:
        if isinstance(other, Operation):
            return other.uuid == self.uuid \
                   and other.product_uuid == self.product_uuid \
                   and other.pose == self.pose \
                   and other.description == self.description
        return False

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self):
        return f"<uuid={self.uuid},product_uuid={self.product_uuid},pose={self.pose},description={self.description}>"



from holour.msg.geometry import Vector3, Pose, Poses
from holour.msg.status import Status
from holour.msg.force_torque import ForceTorque
from holour.msg.script import Script, WaypointConfig
from holour.msg.operation import Operation
from holour.msg.product import Product
from holour.msg.task import Task



import setuptools

with open("README.md", 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="holour",
    version="1.2.1",
    author="Rasmus Lunding",
    author_email="rlh@cs.au.dk",
    description="Collection of common data types",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    url="https://gitlab.au.dk/hrc/common/holour",
    install_requires=[
        'paho-mqtt>=1.5.1',
        'PyYAML>=5.4.1',
    ],
)



from unittest import TestCase

from holour import json_encode, json_decode
from holour.msg import ForceTorque, Vector3


class Test(TestCase):

    def test_force_torque(self):
        force = Vector3(1, 2, 3)
        torque = Vector3(-1.1, -1.2, 1.3)
        ft = ForceTorque(force, torque)
        ft_string = json_encode(ft)

        expected_string = '{"_type": "force_torque", "force": {"_type": "vector3", "x": 1, "y": 2, "z": 3}, "torque": '\
                          '{"_type": "vector3", "x": -1.1, "y": -1.2, "z": 1.3}}'
        assert type(ft_string) == str
        assert ft_string == expected_string, f"Expected: {expected_string}, got: {ft_string}"

        ft_decoded: ForceTorque = json_decode(ft_string)
        assert type(ft_decoded) == ForceTorque, f"Expected object of type {ForceTorque}"
        assert ft_decoded.force == force
        assert ft_decoded.torque == torque
        assert ft_decoded == ft

    def test_equals(self):
        ft1 = ForceTorque(Vector3(1, 2, 3), Vector3(-1.1, -1.2, 1.3))
        ft2 = ForceTorque(Vector3(1, 2, 3), Vector3(-1.1, -1.2, 1.3))
        ft3 = ForceTorque(Vector3(3000, 2, 3), Vector3(-1.1, -1.2, 1.3))

        assert ft1 == ft2
        assert ft1 != ft3
        assert ft1 != "hej"

    def test_ft_repr(self):
        ft = ForceTorque(Vector3(1, 2, 3), Vector3(-1.1, -1.2, 1.3))
        expected, got = f'x=1', f'{ft}'
        assert expected in got, f"Expected {expected} in got: {got}"



from unittest import TestCase
from holour.mqtt import MQTTClient, MQTTConfig


class Test(TestCase):

    def setUp(self) -> None:
        name = 'test_client'
        self.client = MQTTClient(name, MQTTConfig(name))

    def test_mqtt_config(self):
        assert self.client

